# Wwise Connection
UNCONNECTED = 0
CONNECTED = 1
WRONG_CONNECTED = 2

# Page Index
INDEX_PAGE_SFX_IMPORT = 0
INDEX_PAGE_VO_IMPORT = 1
INDEX_MULTILANGUAGE_SILENCE = -1

# Path
PATH_ICON_PROGRAM = 'Icons\\Icon_Program.png'
PATH_ICON_WARNING = 'Icons\\Icon_Warning.png'
PATH_ICON_NOTIFICATION = 'Icons\\Icon_Notification.png'
PATH_ICON_EDIT = 'Icons\\Icon_Edit.png'
PATH_ICON_SUCCESS = 'Icons\\Icon_Success.png'
PATH_ICON_VFY = 'Icons\\Icon_VFY.png'
PATH_ICON_WWISE = {
    "Sound": "Icons/Wwise/Wwise_Sound.png",
    "SwitchContainer": "Icons/Wwise/Wwise_SwitchContainer.png",
    "RandomContainer": "Icons/Wwise/Wwise_RandomContainer.png",
    "SequenceContainer": "Icons/Wwise/Wwise_SequenceContainer.png",
    "BlendContainer": "Icons/Wwise/Wwise_BlendContainer.png"
}

# Wwise Info
WWISE_PROJECT_NAME = ''
WWISE_VERSION = 'Wwise 20xx.x.xx'
WWISE_LANGUAGE = ['Chinese', 'English', 'Korean', 'Japanese']
WWISE_VO_IMPORT_TYPE = ['AI 占位资源', '正式资源']